from argparse import ArgumentParser
from getpass import getpass
from ..sharpai_api import SharpAIFolder,SA_API

from . import BaseSharpAICLICommand
from ..utils.get_id import get_id

class DeviceCommands(BaseSharpAICLICommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        device_parser = parser.add_parser(
            "device", help="{register, unregister} management"
        )
        device_subparsers = device_parser.add_subparsers(
            help="device management command"
        )
        device_register_parser = device_subparsers.add_parser(
            "register", help="register device to sharpai hub"
        )
        device_unregister_parser = device_subparsers.add_parser(
            "unregister", help="unregister device from sharpai hub"
        )

        device_register_parser.set_defaults(func=lambda args: RegisterCommand(args))
        device_unregister_parser.set_defaults(func=lambda args: UnRegisterCommand(args))
class BaseDeviceCommand:
    def __init__(self, args):
        self.args = args
        self._api = SA_API()

class RegisterCommand(BaseDeviceCommand):
    def run(self):
        device_id = get_id().replace(':','')
        SharpAIFolder.save_device_id(device_id)

        self._api.register_device(device_id)
class UnRegisterCommand(BaseDeviceCommand):
    def run(self):
        device_id = None
        device_id = SharpAIFolder.get_device_id()

        if device_id is None:
            device_id = get_id().replace(':','')

        self._api.unregister_device(device_id)




from argparse import ArgumentParser
from getpass import getpass
from ..sharpai_api import SharpAIFolder,SA_API

from . import BaseSharpAICLICommand

class UserCommands(BaseSharpAICLICommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        login_parser = parser.add_parser(
            "login", help="Log in using the same credentials as on sharpai.org"
        )
        login_parser.set_defaults(func=lambda args: LoginCommand(args))
class BaseUserCommand:
    def __init__(self, args):
        self.args = args
        self._api = SA_API()
class LoginCommand(BaseUserCommand):
    def run(self):
        
        print(  # docstyle-ignore
            """
        :'######::'##::::'##::::'###::::'########::'########:::::'###::::'####:
        '##... ##: ##:::: ##:::'## ##::: ##.... ##: ##.... ##:::'## ##:::. ##::
        ##:::..:: ##:::: ##::'##:. ##:: ##:::: ##: ##:::: ##::'##:. ##::: ##::
        . ######:: #########:'##:::. ##: ########:: ########::'##:::. ##:: ##::
        :..... ##: ##.... ##: #########: ##.. ##::: ##.....::: #########:: ##::
        '##::: ##: ##:::: ##: ##.... ##: ##::. ##:: ##:::::::: ##.... ##:: ##::
        . ######:: ##:::: ##: ##:::: ##: ##:::. ##: ##:::::::: ##:::: ##:'####:
        :......:::..:::::..::..:::::..::..:::::..::..:::::::::..:::::..::....::

        """
        )
        username = input("Username: ")
        password = getpass("Password: ")
        response = self._api.login(username,password)
        if response.ok:
            print(f"Login successful, saving token to file: {SharpAIFolder.path_token}")
            json_val = response.json()
            login_token = json_val['data']['authToken']
            userId = json_val['data']['userId']

            SharpAIFolder.save_token(login_token,username,userId)

            userInfo = SharpAIFolder.get_token()
            #print(f'user information {userInfo}')
        else:
            print(f'Login failed, server response: {response.json()}')


import os
import subprocess
import requests
from argparse import ArgumentParser
from shutil import which
from os.path import expanduser
import platform

from . import BaseSharpAICLICommand
from ..sharpai_api import SharpAIFolder,SA_API

class DeepCameraCommands(BaseSharpAICLICommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        deepcamera_parser = parser.add_parser(
            "deepcamera", help="{start,stop} deepcamera app control"
        )
        deepcamera_subparsers = deepcamera_parser.add_subparsers(
            help="device management command"
        )
        deepcamera_start_parser = deepcamera_subparsers.add_parser(
            "start", help="start deepcamera"
        )
        deepcamera_stop_parser = deepcamera_subparsers.add_parser(
            "stop", help="stop deepcamera"
        )

        deepcamera_start_parser.set_defaults(func=lambda args: DeepCameraStartCommand(args))
        deepcamera_stop_parser.set_defaults(func=lambda args: DeepCameraStopCommand(args))
class BaseDeepCameraCommands:
    def __init__(self, args):
        self.runtime_folder = expanduser("~/.sharpai/deepcamera")
        self.args = args
        self._api = SA_API()

    def check_credential(self):
        self.userInfo = SharpAIFolder.get_token()
        if self.userInfo is None:
            print('Please login with command: sharpai_cli login')
            exit(-1)
        device_id = SharpAIFolder.get_device_id()
        if device_id is None:
            print('Please register device with command: sharpai_cli device register')
            exit(-1)

    def check_environment(self):
        self.check_if_docker_installed()
        self.check_if_docker_has_permission()
        self.check_if_docker_compose_installed()

    def check_if_docker_installed(self):
        if which('docker'):
            pass
        else:
            print('docker is not installed, please install docker: https://docs.docker.com/engine/install/')
            exit(-1)
    def check_if_docker_compose_installed(self):
        if which('docker-compose'):
            self.docker_compose_path = which('docker-compose')
        else:
            print('docker-compose is not installed, please install docker-compose: pip install docker-compose')
            exit(-1)
    def check_if_docker_has_permission(self):
        output = subprocess.getoutput("docker ps")
        if 'CONTAINER ' not in output:
            print('The user has no permission to docker, please run following command to assign permission:\n')
            print('1. sudo groupadd docker')
            print('2. sudo usermod -aG docker $USER')
            print('3. newgrp docker')
            print('logout/login your account, if there\'s issue still, please reboot')
            #print(output)
            exit(-1)

class DeepCameraStartCommand(BaseDeepCameraCommands):
    def run(self):
        self.check_environment()
        self.check_credential()

        os.makedirs(self.runtime_folder, exist_ok=True)
        
        processor = platform.processor()
        print(processor)
        arch = None
        if processor == 'x86_64':
            arch = 'x86'
            docker_compose_yml = 'docker-compose-x86.yml'
        elif processor == 'i386':
            arch = 'x86'
            docker_compose_yml = 'docker-compose-x86.yml'
        elif 'Intel64' in processor:
            arch = 'x86'
            docker_compose_yml = 'docker-compose-x86.yml'
        elif processor == 'aarch64':
            if 'tegra' in platform.platform():
                arch = 'aarch64'
                print('Detected you are using Nvidia Jetson, only support jetpack 4.6, please file issue when you face issue.')
                docker_compose_yml = 'docker-compose-l4t-r32.6.1.yml'
            
        if arch == None:
            print('Your platform is not supported, please file an issue on github for feature request: https://github.com/SharpAI/DeepCamera/issues')
            exit(-1)
        yml_url = f'https://raw.githubusercontent.com/SharpAI/applications/main/deepcamera/{docker_compose_yml}'
        env_url = 'https://raw.githubusercontent.com/SharpAI/applications/main/deepcamera/.env'
        
        yml_path = os.path.join(self.runtime_folder,'docker-compose.yml')
        env_path = os.path.join(self.runtime_folder,'.env')
        
        response=requests.get(yml_url)
        open(yml_path, "wb").write(response.content)

        response=requests.get(env_url)
        open(env_path, "wb").write(response.content)

        print('Downloaded the latest docker-compose.yml')
        print('Start to pull latest docker images, it will take a while for the first time')

        command = f'{self.docker_compose_path} -f {yml_path} pull'
        subprocess.getoutput(command)
        
        print('Starting DeepCamera with docker-compose')
        args = [self.docker_compose_path, '-f' , yml_path,'up']
        subprocess.Popen( args= args, cwd=self.runtime_folder)
        
class DeepCameraStopCommand(BaseDeepCameraCommands):
    def run(self):
        self.check_environment()
        
        yml_path = os.path.join(self.runtime_folder,'docker-compose.yml')
        
        args = [self.docker_compose_path, '-f' , yml_path,'down']
        subprocess.Popen( args= args, cwd=self.runtime_folder)




#!/usr/bin/env python


from argparse import ArgumentParser
from sharpai_hub.commands.user import UserCommands
from sharpai_hub.commands.device import DeviceCommands
from sharpai_hub.commands.deepcamera import DeepCameraCommands

def main():
    parser = ArgumentParser(
        "sharpai-cli", usage="sharpai-cli <command> [<args>]"
    )
    commands_parser = parser.add_subparsers(help="sharpai-cli command helpers")

    # Register commands
    UserCommands.register_subcommand(commands_parser)
    DeviceCommands.register_subcommand(commands_parser)
    DeepCameraCommands.register_subcommand(commands_parser)

    # Let's go
    args = parser.parse_args()

    if not hasattr(args, "func"):
        parser.print_help()
        exit(1)

    # Run
    service = args.func(args)
    service.run()


if __name__ == "__main__":
    main()


from gettext import gettext
import os
from os.path import expanduser
import json
from typing import BinaryIO, Dict, Iterable, Iterator, List, Optional, Tuple, Union
import requests

class SA_API:
    def __init__(self):
        self.server='http://dp.sharpai.org:3000'

    def login(self,username,password):
        request_url = self.server + '/api/v1/login'
        result = requests.post(request_url, data={'username': username,'password':password})
        return result

    def get_group_id(self,userInfo):
        request_url = self.server + '/api/v1/groups'
        response = requests.get(request_url, params={'groupName': 'home', 'creator': userInfo['username']})
        if response.ok:
            groupId = response.json()['_id']
            SharpAIFolder.save_group_id(groupId)
            return groupId
        print(f'Get group ID failed, server response {response.json()}')
        return None

    def register_device(self,deviceId):
        userInfo = SharpAIFolder.get_token()
        if userInfo is None:
            print('Please login first: sharpai_cli login')
            return
        group_id = self.get_group_id(userInfo)

        request_url = self.server + '/api/v1/groups/'+group_id+'/devices'
        headers = {
            'X-User-Id': userInfo['userId'],
            'X-Auth-Token': userInfo['token']
        }
        device_name = input('Please input the device name:')
        data = {
            'uuid':deviceId,
            'deviceName': device_name,
            'type':'in'
        }
        response = requests.post(request_url,headers=headers,data=data)

        if response.ok:
            print('Device successfully added')
        else:
            print(f'{response.status_code}')
            print(f'Add device failed, server response: {response.content}')

    def unregister_device(self,device_id):
        userInfo = SharpAIFolder.get_token()
        if userInfo is None:
            print('Please login first: sharpai_cli login')
            return
        request_url = self.server + '/api/v1/devices/' + device_id
        headers = {
            'X-User-Id': userInfo['userId'],
            'X-Auth-Token': userInfo['token']
        }

        response = requests.delete(request_url,headers=headers)

        if response.ok:
            print('Device successfully deleted')
            print(f'{response.json()}')
        else:
            print(f'{response.status_code}')
            print(f'Delete device failed, server response: {response.content}')
        
class SharpAIFolder:
    path_token = expanduser("~/.sharpai/token.json")
    path_device_id = expanduser("~/.sharpai/device_id")
    path_group_id = expanduser("~/.sharpai/group_id")
    path_token_env = expanduser("~/.sharpai/token.env")
    @classmethod
    def save_group_id(cls, groupId):
        """
        Save token, creating folder as needed.

        Args:
            deviceId (`str`):
                The device to save to the [`SharpAIFolder`]
        """
        os.makedirs(os.path.dirname(cls.path_group_id), exist_ok=True)

        with open(cls.path_group_id, "w+") as f:
            f.write(groupId)
    @classmethod
    def save_device_id(cls, deviceId):
        """
        Save token, creating folder as needed.

        Args:
            deviceId (`str`):
                The device to save to the [`SharpAIFolder`]
        """
        os.makedirs(os.path.dirname(cls.path_device_id), exist_ok=True)

        with open(cls.path_device_id, "w+") as f:
            f.write(deviceId)
    @classmethod
    def get_device_id(cls):
        """
        Save token, creating folder as needed.

        Args:
            deviceId (`str`):
                The device to save to the [`SharpAIFolder`]
        """
        device_id = None
        try:
            with open(cls.path_device_id, "r") as f:
                device_id = f.read()
        except:
            pass
        return device_id

    @classmethod
    def save_token(cls, token, username, userId):
        """
        Save token, creating folder as needed.

        Args:
            token (`str`):
            username (`str`):
            userId (`str`):
                The user credentials to save to the [`SharpAIFolder`]
        """
        os.makedirs(os.path.dirname(cls.path_token), exist_ok=True)

        userInfo = {
            'username':username,
            'userId':userId,
            'token':token
        }
        with open(cls.path_token, "w+") as f:
            json.dump(userInfo,f)
        tokenEnv = 'SHARP_AI_TOKEN={}\nSHARP_AI_USER_ID={}\nSHARP_AI_USERNAME={}\n'.format(token,userId,username)
        with open(cls.path_token_env, "w+") as f:
            f.write(tokenEnv)

    @classmethod
    def get_token(cls) -> Optional[Dict]:
        """
        Get token or None if not existent.

        Returns:
            `Dict` or `None`: The token, `None` if it doesn't exist.

        """
        userInfo = None
        try:
            with open(cls.path_token, "r") as f:
                userInfo = json.load(f)
        except FileNotFoundError:
            pass
        return userInfo

    @classmethod
    def delete_token(cls):
        """
        Deletes the token from storage. Does not fail if token does not exist.
        """
        try:
            os.remove(cls.path_token)
        except FileNotFoundError:
            pass




from setuptools import find_packages, setup


def get_version() -> str:
    return "0.1.13"


install_requires = [
    "requests",
    "tqdm",
    "getmac>=0.8.3",
    "pyyaml>=5.1",
    "importlib_metadata;python_version<'3.8'",
    "packaging>=20.9",
]

extras = {}

setup(
    name="sharpai_hub",
    version=get_version(),
    author="SharpAI LLC",
    author_email="simba@sharpai.org",
    description=(
        "Client library to download application from sharpai hub"
    ),
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    keywords=(
        "model-hub machine-learning models natural-language-processing deep-learning"
        " Edge AI"
    ),
    license="MIT",
    url="https://github.com/SharpAI/sharpai_hub",
    package_dir={"": "src"},
    packages=find_packages("src"),
    extras_require=extras,
    entry_points={
        "console_scripts": [
            "sharpai-cli=sharpai_hub.cli:main"
        ]
    },
    python_requires=">=3.6.0",
    install_requires=install_requires,
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
)
